-----------------------------------------------------------------------------------------------
-- Client Lua Script for FrozenPortraits. By lyravega, Phare@Ravenous
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------

local FrozenPortraits = {}
 
function FrozenPortraits:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 
    
    o.mode = 1
    o.portraits = {}

    return o
end

function FrozenPortraits:Init()
	Apollo.RegisterAddon(self)
	Apollo.RegisterSlashCommand("fp", "OnToggleFrozenPortraits", self)
	Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", "OnInterfaceMenuListHasLoaded", self)
	Apollo.RegisterEventHandler("ToggleFrozenPortraits", "OnToggleFrozenPortraits", self)
end

function FrozenPortraits:OnSave(saveDepth)
	if saveDepth ~= GameLib.CodeEnumAddonSaveLevel.Character then return nil end

	local savedVariables = {}
	savedVariables.mode = self.mode

	return savedVariables
end

function FrozenPortraits:OnRestore(saveDepth, savedVariables)
	if saveDepth ~= GameLib.CodeEnumAddonSaveLevel.Character then return end	
	if savedVariables == {} or savedVariables == nil then return end
	
	self.mode = savedVariables.mode or self.mode
end

function FrozenPortraits:GetAsyncLoadStatus()
	local targetFrame = Apollo.GetAddon("TargetFrame")
	
	if not targetFrame or not targetFrame.luaFocusFrame or not targetFrame.luaTargetFrame or not targetFrame.luaUnitFrame then
		return Apollo.AddonLoadStatus.Loading
	end
	
	local N, portraits = 1, self.portraits
	
	for i = 1, 5 do
		if i == 1 then
			portraits[N] = targetFrame.luaFocusFrame.arClusterFrames[1]:FindChild("TotFrame"):FindChild("TargetModel"); N = N + 1
			portraits[N] = targetFrame.luaTargetFrame.arClusterFrames[1]:FindChild("TotFrame"):FindChild("TargetModel"); N = N + 1 
			portraits[N] = targetFrame.luaUnitFrame.arClusterFrames[1]:FindChild("TotFrame"):FindChild("TargetModel"); N = N + 1 
		end
		portraits[N] = targetFrame.luaFocusFrame.arClusterFrames[i]:FindChild("TargetModel"); N = N + 1
		portraits[N] = targetFrame.luaTargetFrame.arClusterFrames[i]:FindChild("TargetModel"); N = N + 1 
		portraits[N] = targetFrame.luaUnitFrame.arClusterFrames[i]:FindChild("TargetModel"); N = N + 1 
	end
	
	self:Freeze(); return Apollo.AddonLoadStatus.Loaded
end

function FrozenPortraits:Freeze()
	local mode, animate, show = self.mode, true, true
	
	if mode == 1 then animate, show = false, true
	elseif mode == 2 then animate, show = true, false
	elseif mode == 3 then animate, show = false, false end
		
	for _,portrait in next, self.portraits do
		portrait:SetAnimated(animate); portrait:Show(show)
	end
end

function FrozenPortraits:OnInterfaceMenuListHasLoaded()
	Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "FrozenPortraits", {"ToggleFrozenPortraits", "", "IconSprites:Icon_Windows_UI_CRB_FieldStudy_Sleeping"})
	self:UpdateInterfaceMenuAlerts()
end

function FrozenPortraits:UpdateInterfaceMenuAlerts()
	local credits = "by lyravega, Phare@Ravenous\n"
	local status = self.mode == 1 and "Frozen" or self.mode == 2 and "Hidden" or self.mode == 3 and "Frozen and Hidden" or "Normal"
	
	Event_FireGenericEvent("InterfaceMenuList_AlertAddOn", "FrozenPortraits", {false, credits..status, self.mode})
end

function FrozenPortraits:OnToggleFrozenPortraits()	
	self.mode = self.mode < 3 and self.mode + 1 or 0
	self:UpdateInterfaceMenuAlerts(); self:Freeze()
	
	local message = self.mode == 1 and "Frozen" or self.mode == 2 and "Hidden" or self.mode == 3 and "Frozen and Hidden" or "Normal"
	
	ChatSystemLib.PostOnChannel( ChatSystemLib.ChatChannel_Datachron, message, "Portraits:" )
end

local FrozenPortraitsInst = FrozenPortraits:new()
FrozenPortraitsInst:Init()